/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.endpoint.action;

import it.eng.console.process.bo.ProcessBO;
import it.eng.console.util.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.action.AbstractAction;
import it.eng.spago.tracing.TracerSingleton;

import java.util.Set;

import org.spagic.metadb.model.Property;
import org.spagic.metadb.model.PropertyStep;
import org.spagic.metadb.model.Step;

public class StepDetailAction extends AbstractAction {

	private static final long serialVersionUID = 1L;

	public void service(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception {
		String stepName = (String)serviceRequest.getAttribute("stepId");
		SourceBean stepBean = createDetailBean(stepName);
		SourceBean stepPropertiesBean = createPropertiesBean(stepName);
		stepBean.setAttribute(stepPropertiesBean);
		serviceResponse.setAttribute(stepBean);
	}
	
	protected SourceBean createDetailBean(String stepName) {
		SourceBean stepBean = null;
		try {
			stepBean = new SourceBean("DETAIL");
			Step stepDetail = ProcessBO.getStepDetail(stepName);
			stepBean.setAttribute("nome", stepDetail.getName());
			stepBean.setAttribute("id", stepDetail.getIdStep());
			stepBean.setAttribute("descrizione", stepDetail.getDescription());
			stepBean.setAttribute("namespace", stepDetail.getNamespace());
		}catch(SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "StepDetailAction::createDetailBean:: ", sbe);
		}
		return stepBean;
	}
	
	protected SourceBean createPropertiesBean(String stepName) {
		SourceBean stepPropertiesBean = null;
		try {
			stepPropertiesBean = new SourceBean("PROPERTIES");
			Set<PropertyStep> properties = ProcessBO.getStepProperties(stepName);
			Property property = null;
			SourceBean propertyBean = null;
			String propertyValue = "";
			for (PropertyStep ps : properties) {
				property = ps.getProperty();
				propertyBean = new SourceBean("PROPERTY");
				propertyBean.setAttribute("name", property.getCode());
				propertyValue = (property.getValue() != null) 
								? property.getValue()
								: "";								
				propertyBean.setAttribute("value", propertyValue);
				stepPropertiesBean.setAttribute(propertyBean);
			}
		}catch(SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "StepDetailAction::createPropertiesBean:: ", sbe);
		}
		return stepPropertiesBean;
	}

}
