/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.process.action;

import java.text.DateFormat;
import java.util.List;

import org.hibernate.Criteria;
import org.spagic.metadb.base.ProcessInstanceStateConstants;
import org.spagic.metadb.model.Process;

import it.eng.console.audit.action.ListAction;
import it.eng.console.audit.bo.ProcessInstanceBO;
import it.eng.console.hibernate.pagination.CriteriaHibernatePage;
import it.eng.console.process.bo.ProcessBO;
import it.eng.console.util.Constants;
import it.eng.console.util.DateFormatter;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.tracing.TracerSingleton;

public class ProcessListFilteredAction extends ListAction {
	protected String _taxonomyName = null;
	protected String _taxonomyValue = null;
	protected String _idSelectedNode = null;			
	protected String _iterType = null;	
	private String _orderBy = null;
	private String _orderType = null;
	
	public void service(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception {		
		SessionContainer sessionContainer = getRequestContainer().getSessionContainer();
		int page = getRequestPage(serviceRequest, sessionContainer, Constants.PAGE_PROCESS);
		
		//get Parameters from request
		loadParametersFromRequest(serviceRequest);
		
		//save into response for pagination
		SourceBean processBeanList = new SourceBean("LIST");
		saveParametersForPagination(processBeanList);
	
		//call ProcessBO 
		List processList = null;
		Criteria criteria = null;
		CriteriaHibernatePage hibernatePage = null;
		if(_idSelectedNode!=null) 
			criteria = ProcessBO.getProcessListByTaxonomy(_taxonomyName, _taxonomyValue, _orderBy, _orderType);
		else if (_iterType!=null)
			criteria = ProcessBO.getProcessListByPool((new Long(_iterType)).longValue(), _orderBy, _orderType);
		else
			criteria = ProcessBO.getProcessList(_orderBy, _orderType);
		hibernatePage = new CriteriaHibernatePage(criteria, page, Constants.sizeProcessList);
		
		processList = hibernatePage.getList();
		
		//create response
		Process process = null;		
		String jmxStatus;
		String serviceAssemblyName = "";
		
		for(int indexProcess=0; indexProcess<processList.size(); indexProcess++) {
			process = (Process)processList.get(indexProcess);
			if (process!=null) 
				processBeanList.setAttribute( createProcessBean(process, _iterType, sessionContainer));
		}
		serviceResponse.setAttribute(processBeanList);
	}
	
	private void loadParametersFromRequest(SourceBean request) {
		_taxonomyName = (String)request.getAttribute(Constants.LBL_SELECTED_TAXONOMY);
		_taxonomyValue = (String)request.getAttribute(Constants.LBL_SELECTED_TAXONOMY_VALUE);
		_idSelectedNode = (String)request.getAttribute(Constants.LBL_SELECTED_NODE);			
		_iterType = (String)request.getAttribute(Constants.LBL_SELECTED_ITER);
		_orderBy = (String)request.getAttribute(Constants.LBL_ORDINAMENTO);
		_orderType = (String)request.getAttribute(Constants.LBL_TIPO_ORDINAMENTO);

	}
	
	private void saveParametersForPagination(SourceBean processBeanList) throws SourceBeanException {
		try {
			if(_idSelectedNode!=null)
				processBeanList.setAttribute(Constants.LBL_SELECTED_NODE, _idSelectedNode);
			if(_iterType!=null)
				processBeanList.setAttribute(Constants.LBL_SELECTED_ITER, _iterType);
			if(_taxonomyName!=null)
				processBeanList.setAttribute(Constants.LBL_SELECTED_TAXONOMY, _taxonomyName);
			if(_taxonomyValue!=null)
				processBeanList.setAttribute(Constants.LBL_SELECTED_TAXONOMY_VALUE, _taxonomyValue);
			if(_orderBy!=null)
				processBeanList.setAttribute(Constants.LBL_ORDINAMENTO, _orderBy);
			if(_orderType!=null)
				processBeanList.setAttribute(Constants.LBL_TIPO_ORDINAMENTO, _orderType);
		} catch(SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessListFilteredAction::saveParametersForPagination: ", sbe);
		}
	}
	
	protected SourceBean createProcessBean(Process process, String iterType, SessionContainer sessionContainer) 
		throws SourceBeanException {		
		SourceBean item = null;
		long idProcess;
		try {
			item = new SourceBean("ITEM");
			idProcess = process.getIdProcess();
			item.setAttribute("id", idProcess);
			item.setAttribute("name", process.getName());				
						
			DateFormat formatter = DateFormatter.getShortDateFormat(sessionContainer);
			String dateTemp = formatter.format(process.getStartdate());					
			item.setAttribute("dataInizio", dateTemp);				
			dateTemp = (process.getEnddate() != null) ? formatter.format(process.getEnddate()) : "";					
			item.setAttribute("dataFine", dateTemp);
			
			String version = String.valueOf(process.getProcessVersion());
			item.setAttribute("versione", version);	
			if (iterType == null) 
				iterType = Long.toString(Constants.ALL_ITER);
			item.setAttribute("istanze", Long.toString(ProcessInstanceBO.getCountProcessInstances(idProcess, Long.parseLong(iterType))));
			item.setAttribute("istanzeAttive", Long.toString(ProcessInstanceBO.getCountProcessInstancesByState(idProcess, ProcessInstanceStateConstants.PROCESS_ACTIVE, null)));
			item.setAttribute("istanzeInErrore", Long.toString(ProcessInstanceBO.getCountProcessInstancesByState(idProcess, ProcessInstanceStateConstants.PROCESS_FAULTED, null)));
			item.setAttribute("istanzeEseguite", Long.toString(ProcessInstanceBO.getCountProcessInstancesByState(idProcess, ProcessInstanceStateConstants.PROCESS_DONE, null)));
			String type = process.getTypeService().getValue();
			item.setAttribute("type", type);
			
			item.setAttribute("isMonitorEnabled",process.isMonitorEnabled());
		} catch(SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessListAction::createProcessBean:: ", sbe);
			throw sbe;
		}
		
		return item;
	}
		
}
