/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.tag;

import it.eng.console.util.Constants;
import it.eng.spago.base.RequestContainerAccess;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.tracing.TracerSingleton;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

import org.spagic.metadb.base.ProcessInstanceStateConstants;

public class ProcessInstanceListTag extends TagSupport {
	private boolean _advancedSearch = false;
	private boolean _restart = false;
	private boolean _showProcessDetail = true;
	private String listPath = null;
	
    public int doStartTag() throws JspException {
        TracerSingleton.log(Constants.project, TracerSingleton.DEBUG, "ProcessInstanceListTag::doStartTag:: invocato");
        if(pageContext == null) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL,
                "ProcessInstanceListTag::doStartTag:: pageContext nullo");
            throw new JspException("pageContext nullo");
        } 
        HttpServletRequest httpRequest = null;
        try {
            httpRequest = (HttpServletRequest) pageContext.getRequest();
        } catch(Exception ex) {
            TracerSingleton.log(
            		Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListTag::doStartTag::", ex);
            throw new JspException(ex.getMessage());
        } 
        if(httpRequest == null) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListTag::doStartTag:: httpRequest nullo");
            throw new JspException("httpRequest nullo");
        }
        SessionContainer sessionContainer =
            RequestContainerAccess.getRequestContainer(httpRequest).getSessionContainer();
        //variabili per la gestione selettiva delle ripartenze
        boolean selectAll = true;;
        ArrayList differentBehaviour = null;
        if(_restart){
	        selectAll = (Boolean)sessionContainer.getAttribute(Constants.SELECT_ALL);
	        differentBehaviour = (ArrayList)sessionContainer.getAttribute(Constants.DIFFERENT_BEHAVIOUR);
        }
        
        ResponseContainer responseContainer =
            ResponseContainerAccess.getResponseContainer(httpRequest);
        if(responseContainer == null) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListTag::doStartTag:: httpRequest nullo");
            throw new JspException("responseContainer nullo");
        }    
        StringBuffer output = new StringBuffer(); 
       
        SourceBean listBean = (SourceBean)responseContainer.getServiceResponse().getAttribute(getListPath());
        List processList = (listBean != null) ? listBean.getAttributeAsList("ITEM") : null;
		TracerSingleton.log(Constants.project, TracerSingleton.DEBUG, "ProcessInstanceListTag::doStartTag:: serviceResponse is:" + responseContainer.getServiceResponse());
		
		if((processList != null) && (processList.size() > 0)) {
			TracerSingleton.log(Constants.project, TracerSingleton.DEBUG, "ProcessInstanceListTag::doStartTag:: processList.size: " + processList.size());

			SourceBean processInstance;
			Long processInstanceId, processId;
			String process = (listBean != null) ? (String)listBean.getAttribute(Constants.LBL_PROCESS_ID) : null;
			for(int indexProcess=0; indexProcess<processList.size();indexProcess++) {
				try {
					processInstance = (SourceBean)processList.get(indexProcess);
					processInstanceId = (Long)processInstance.getAttribute("id"); 
					processId = (Long)processInstance.getAttribute("idProcesso");
					output.append("<tr>\n");	
					if(_restart) {
						output.append("<td><input type=\"checkbox\" ");
						if((selectAll && !differentBehaviour.contains(processInstanceId))
							|| (!selectAll && differentBehaviour.contains(processInstanceId))){
							output.append(" checked ");
							output.append(" value=\"on\"");
						}
						else
							output.append(" value=\"off\"");
						output.append(" name=\""+ Constants.PREFIX_CHECKBOX + processInstanceId + "\"");
						output.append(" onclick=\"if(this.value=='off') this.value='on'; else this.value='off';\"></td>\n");
					}
					output.append("<td>" + processInstanceId + "</td>\n");
					output.append("<td>" + (String)processInstance.getAttribute("processo") + "</td>\n");
					output.append("<td>" + (String)processInstance.getAttribute("iter") + "&nbsp;</td>\n");
					output.append("<td>" + (String)processInstance.getAttribute("iterIstanza") + "&nbsp;</td>\n");
					if (isAdvancedSearch())
						output.append("<td>" + (String)processInstance.getAttribute("owner") + "&nbsp;</td>\n");
					output.append("<td>" + (String)processInstance.getAttribute("dataInizio") + "</td>\n");
					output.append("<td>" + (String)processInstance.getAttribute("dataFine") + "&nbsp;</td>\n");
					
					int stato = ((Integer)processInstance.getAttribute("stato")).intValue();
					if(stato == ProcessInstanceStateConstants.PROCESS_ACTIVE){
						output.append("<td><img src=\"" + httpRequest.getContextPath() + "/images/console/wait.gif\" alt=\"Running\"/>");
					}else if(stato == ProcessInstanceStateConstants.PROCESS_DONE){ 
						output.append("<td><img src=\"" + httpRequest.getContextPath() + "/images/console/executed.gif\" alt=\"Executed\"/>");	
					}else if ((stato == ProcessInstanceStateConstants.PROCESS_EXPIRED) || (stato == ProcessInstanceStateConstants.PROCESS_KILLED)){ 
						output.append("<td><img src=\"" + httpRequest.getContextPath() + "/images/console/clock_red.png\" alt=\"Killed\"/>");
					}else{
						output.append("<td><img src=\"" + httpRequest.getContextPath() + "/images/console/fault.gif\" alt=\"Faulted\"/>");
					}
					if ("TRUE".equals(processInstance.getAttribute("restarted"))) {
						output.append("<img src=\"" + httpRequest.getContextPath() + "/images/console/lightning.png\" alt=\"Restarted\"/>");	
					}
					output.append("</td>\n");	
					
					if (isShowProcessDetail() || (!isShowProcessDetail() && process == null)){
						String advancedSearch = isAdvancedSearch()? "TRUE" : "FALSE";
						output.append("<td><a href=\"#\" onclick=\"showProcessInstanceGraph('" + processInstanceId + "', '" + processId + "','" + advancedSearch +"');return false;\"><img src=\"" + httpRequest.getContextPath() + "/images/console/graph.gif\"/></a></td>\n");
					} else 
						output.append("<td></td>");
					output.append("</tr>\n");
				} catch(ClassCastException cce) {
					TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListTag::doStartTag::", cce);
				}
			}
		} else {//generate a row empty
			TracerSingleton.log(Constants.project, TracerSingleton.DEBUG, "ProcessInstanceListTag::doStartTag:: processList.size: 0");
			output.append("<tr>\n");			
			output.append("<td></td>\n");
			output.append("<td></td>\n");
			output.append("<td></td>\n");
			output.append("<td></td>\n");
			if (isAdvancedSearch())
				output.append("<td></td>\n");
			output.append("<td></td>\n");
			output.append("<td></td>\n");
			output.append("<td></td>\n");
			output.append("<td></td>");
			output.append("</tr>\n");
		}
		
        try {
            pageContext.getOut().print(output.toString());
        } catch(Exception ex) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListTag::doStartTag::", ex);
            throw new JspException(ex.getMessage());
        } 
        return SKIP_BODY;
    }
    
	public boolean isAdvancedSearch() {
		return _advancedSearch;
	}

	public void setAdvancedSearch(boolean advancedSearch) {
		_advancedSearch = advancedSearch;
	}

	public boolean isRestart() {
		return _restart;
	}

	public void setRestart(boolean restart) {
		_restart = restart;
	}	
	public boolean isShowProcessDetail() {
		return _showProcessDetail;
	}

	public void setShowProcessDetail(boolean processDetail) {
		_showProcessDetail = processDetail;
	}
	
	public String getListPath() {
		return listPath;
	}

	public void setListPath(String listPath) {
		this.listPath = listPath;
	}

	/**
     * @see javax.servlet.jsp.tagext.Tag#doEndTag()
     */
    public int doEndTag() throws JspException {
        TracerSingleton.log(Constants.project, TracerSingleton.DEBUG, "ProcessInstanceListTag::doEndTag:: invocato");
        return super.doEndTag();
    } 

}
