/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.jmx.bo;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import it.eng.console.util.Constants;
import it.eng.jmx.beans.SummaryBean;
import it.eng.jmx.util.JmxConnector;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

import javax.management.MBeanAttributeInfo;
import javax.management.MBeanServer;
import javax.management.MBeanServerConnection;
import javax.management.ObjectInstance;
import javax.management.ObjectName;
import javax.management.openmbean.CompositeData;

import org.apache.servicemix.jbi.framework.ServiceAssemblyLifeCycle;
import org.apache.servicemix.jbi.framework.ServiceAssemblyMBean;

import sun.management.ConnectorAddressLink;

public class JmxBO {

	public static SummaryBean getSummary(String idServer) {
		SummaryBean summary = new SummaryBean();
		try {
			JmxConnector connector = (new JmxConnector(idServer));
			MBeanServerConnection connection = connector.getJmxConnector().getMBeanServerConnection();
			ObjectName thread = new ObjectName("java.lang:type=Threading");
			ObjectName classes = new ObjectName("java.lang:type=ClassLoading");
			ObjectName memory = new ObjectName("java.lang:type=Memory");
		    summary.setPeak( ((Integer)connection.getAttribute(thread,"PeakThreadCount")).intValue()); 
		    summary.setDaemonThreads(((Integer)connection.getAttribute(thread,"DaemonThreadCount")).intValue());    
		    summary.setTotalThreads(((Long)connection.getAttribute(thread,"TotalStartedThreadCount")).longValue());
		    summary.setLiveThreads(((Integer)connection.getAttribute(thread,"ThreadCount")).intValue());
		    summary.setCurrentClassesLoaded(((Integer)connection.getAttribute(classes,"LoadedClassCount")).intValue());
		    summary.setTotalClassesLoaded(((Long)connection.getAttribute(classes,"TotalLoadedClassCount")).longValue());
		    CompositeData compositeData = (CompositeData)connection.getAttribute(memory,"HeapMemoryUsage");
		    summary.setCurrentHeapSize((Long)compositeData.get("used")/1024);
		    summary.setMaxHeapSize((Long)compositeData.get("max")/1024);
		    connector.close();
		} catch (Exception ex) {
           TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "JmxBO::getSummary: ", ex);
        } 
        return summary;
	}
	
	public static String[] getQueueAttributes(String idServer) {
		return getQueueAttributes(idServer,"org.apache.activemq:BrokerName=localhost,Type=Queue,*");
	}
	
	public static String[] getTopicAttributes(String idServer) {
		return getQueueAttributes(idServer,"org.apache.activemq:BrokerName=localhost,Type=Topic,*");
	}
	
	public static String[] getQueueAttributes(String idServer, String labelObjectName) {
		String[] attributesName = null;
		Set queueObjectNameList = null;
		try {
			JmxConnector connector = (new JmxConnector(idServer));
			MBeanServerConnection connection = connector.getJmxConnector().getMBeanServerConnection();
			queueObjectNameList = connection.queryNames(new ObjectName("org.apache.activemq:BrokerName=localhost,Type=Queue,*"),null);
			ObjectName objName = (ObjectName) queueObjectNameList.iterator().next();
			MBeanAttributeInfo[] infoMBeanAttributes = connection.getMBeanInfo(objName).getAttributes();
			attributesName = new String[infoMBeanAttributes.length];
			int indexAttribute;
			for (indexAttribute = 0; indexAttribute < infoMBeanAttributes.length; indexAttribute++) 
				attributesName[indexAttribute] = infoMBeanAttributes[indexAttribute].getName();
			
			connector.close();
		} catch (Exception ex) {
           TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "JmxBO::getQueue(String idServer, String labelObjectName): ", ex);
        } 
		return attributesName;
	}
	
	public static List getQueue(String idServer) throws Exception {
        return getQueue(idServer, "org.apache.activemq:BrokerName=localhost,Type=Queue,*");
	}

	public static List getTopic(String idServer) throws Exception {
		return getQueue(idServer, "org.apache.activemq:BrokerName=localhost,Type=Topic,*");
	}
	
	public static List getQueue(String idServer, String labelObjectName) throws Exception {
		String[] attributes = getQueueAttributes(idServer, labelObjectName);
		List queueValueList = new ArrayList();
		
		Set queueObjectNameList = null;
		try {
			JmxConnector connector = (new JmxConnector(idServer));
			MBeanServerConnection connection = connector.getJmxConnector().getMBeanServerConnection();
			queueObjectNameList = connection.queryNames(new ObjectName(labelObjectName),null);
			ObjectName objName = null;
			for (Iterator i = queueObjectNameList.iterator(); i.hasNext(); ) {
				objName = (ObjectName) i.next();
				queueValueList.add(connection.getAttributes(objName, attributes));
			}
	        connector.close();
		} catch (Exception ex) {
           TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "JmxBO::getQueue(String idServer, String labelObjectName): ", ex);
           throw ex;
        } 
        return queueValueList;
	}
	
	public static String isStarted (String serviceAssemblyName) throws Exception {
		List servers = ConfigSingleton.getInstance().getAttributeAsList("SERVERS.SERVER");
		Iterator itServer = servers.iterator();
		Set serviceAssemblies = null;
		boolean deployed = false;
		String currentState = "";
		JmxConnector connector = null;
		String idServer = "";
		while(itServer.hasNext()){
			idServer = (String)(((SourceBean)itServer.next()).getAttribute("id"));
			try {
				connector = (new JmxConnector(idServer));
				MBeanServerConnection connection = connector.getJmxConnector().getMBeanServerConnection();
				serviceAssemblies = connection.queryNames(new ObjectName("org.apache.servicemix:ContainerName=ServiceMix,Type=ServiceAssembly,Name="+serviceAssemblyName), null);
				if (serviceAssemblies.isEmpty())
					continue;
				deployed = true;
				ObjectName name = (ObjectName)serviceAssemblies.iterator().next();
				currentState = (String)connection.getAttribute(name, it.eng.jmx.util.Constants.CURRENT_STATE);
		        connector.close();
		        break;
			} catch (IOException ioe) {
				TracerSingleton.log(Constants.project, TracerSingleton.INFORMATION, "JmxBO::getQueue(String idServer, String labelObjectName): ", ioe);
				continue;
			} catch (Exception ex) {
	           TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "JmxBO::getQueue(String idServer, String labelObjectName): ", ex);
	           connector.close();
	           throw ex;
	        } 
		}
		if(deployed)
			return currentState;
		else
			return it.eng.jmx.util.Constants.NOT_DEPLOYED;
	}
	
	public static String changeStatus (String serviceAssemblyName, String oldStatus) throws Exception {
		List servers = ConfigSingleton.getInstance().getAttributeAsList("SERVERS.SERVER");
		Iterator itServer = servers.iterator();
		Set serviceAssemblies = null;
		String newState = "";
		boolean deployed = false;
		JmxConnector connector = null;
		String idServer = "";
		while(itServer.hasNext()){
			idServer = (String)(((SourceBean)itServer.next()).getAttribute("id"));
			try {
				connector = (new JmxConnector(idServer));
				MBeanServerConnection connection = connector.getJmxConnector().getMBeanServerConnection();
				serviceAssemblies = connection.queryNames(new ObjectName("org.apache.servicemix:ContainerName=ServiceMix,Type=ServiceAssembly,Name="+serviceAssemblyName), null);
				if (serviceAssemblies.isEmpty()) 
					continue;
				deployed = true;
				ObjectName name = (ObjectName)serviceAssemblies.iterator().next();
		        if (oldStatus.equals(it.eng.jmx.util.Constants.STOPPED)) {
		        	connection.invoke(name, "start", null, null);
		        	connector.close();
		        	newState = it.eng.jmx.util.Constants.STARTED;
		        } else {
		        	connection.invoke(name, "stop", null, null);
		        	connector.close();
		        	newState = it.eng.jmx.util.Constants.STOPPED;
		        }
			} catch (IOException ioe) {
				TracerSingleton.log(Constants.project, TracerSingleton.INFORMATION, "JmxBO::getQueue(String idServer, String labelObjectName): ", ioe);
				throw ioe;
			} catch (Exception ex) {
	           TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "JmxBO::getQueue(String idServer, String labelObjectName): ", ex);
	           connector.close();
	           throw ex;
	        } 
		}
		if (deployed)
			return newState;
		else
			return it.eng.jmx.util.Constants.NOT_DEPLOYED;
	}
}
